Snex.Error exception (Snex v0.4.1)

Copy Markdown View Source

Domain-specific errors returned by Snex.

Summary

Types

The error code indicating communication with the interpreter GenServer failed.

The error code indicating a key is not found in the Snex.Env.t/0.

The error code indicating an environment referenced by the passed in Snex.Env.t/0 is not found in the Python interpreter that ran the command.

Error codes for all errors.

The error code indicating the init script timed out.

The error code for an internal error of Snex.

The error code indicating the interpreter exited with a non-zero status.

The error code for a runtime error.

The error code indicating the response was not received within the timeout.

t()

The type of Snex.Error.

Types

call_failed()

@type call_failed() :: :call_failed

The error code indicating communication with the interpreter GenServer failed.

env_key_not_found()

@type env_key_not_found() :: :env_key_not_found

The error code indicating a key is not found in the Snex.Env.t/0.

env_not_found()

@type env_not_found() :: :env_not_found

The error code indicating an environment referenced by the passed in Snex.Env.t/0 is not found in the Python interpreter that ran the command.

error_code()

Error codes for all errors.

init_script_timeout()

@type init_script_timeout() :: :init_script_timeout

The error code indicating the init script timed out.

internal_error()

@type internal_error() :: :internal_error

The error code for an internal error of Snex.

interpreter_exited()

@type interpreter_exited() :: :interpreter_exited

The error code indicating the interpreter exited with a non-zero status.

python_runtime_error()

@type python_runtime_error() :: :python_runtime_error

The error code for a runtime error.

response_timeout()

@type response_timeout() :: :response_timeout

The error code indicating the response was not received within the timeout.

t()

@type t() :: %Snex.Error{
  __exception__: true,
  code: error_code(),
  reason: String.t() | term(),
  traceback: term()
}

The type of Snex.Error.